@@ -388,7 +388,7 @@ struct msgb *ipa_ccm_make_id_resp(const struct ipaccess_unit *dev,
 		tag = msgb_put(msg, 3 + strlen(str) + 1);
 		tag[0] = 0x00;
 		tag[1] = 1 + strlen(str) + 1;
-		tag[2] = ies_req[1];
+		tag[2] = ies_req[i];
 		memcpy(tag + 3, str, strlen(str) + 1);
 	}
 	ipa_prepend_header(msg, IPAC_PROTO_IPACCESS);
@@ -412,10 +412,14 @@ struct msgb *ipa_ccm_make_id_resp_from_req(const struct ipaccess_unit *dev,
 	/* build a array of the IEIs */
 	while (len >= 2) {
 		uint8_t t_len, t_tag;
-		len -= 2;
+		len -= 2; /* subtract the length of the two bytes read below */
 		t_len = *cur++;
 		t_tag = *cur++;
 
+		/* as the 'tag' is included in the length of t_len, this cannot happen */
+		if (t_len == 0)
+			break;
+
 		if (t_len > len + 1) {
 			LOGP(DLINP, LOGL_ERROR, "IPA CCM tag 0x%02x does not fit\n", t_tag);
 			break;
@@ -423,13 +427,14 @@ struct msgb *ipa_ccm_make_id_resp_from_req(const struct ipaccess_unit *dev,
 
 		ies[num_ies++] = t_tag;
 
-		cur += t_len;
+		/* we need to subtract one from t_len to account for the tag */
+		cur += t_len - 1;
 		/* prevent any unsigned integer underflow due to somebody sending us
 		 * messages with wrong length values */
 		if (len <= t_len)
-			len -= t_len;
-		else
 			len = 0;
+		else
+			len -= t_len - 1;
 	}
 	return ipa_ccm_make_id_resp(dev, ies, num_ies);
 }
@@ -473,7 +478,7 @@ int ipa_ccm_rcvmsg_base(struct msgb *msg, struct osmo_fd *bfd)
 	case IPAC_MSGT_PING:
 		ret = ipa_ccm_send_pong(bfd->fd);
 		if (ret < 0) {
-			LOGP(DLINP, LOGL_ERROR, "Cannot send PING "
+			LOGP(DLINP, LOGL_ERROR, "Cannot send PONG "
 			     "message. Reason: %s\n", strerror(errno));
 			break;
 		}
