@@ -19,7 +19,7 @@
 #include <osmocom/core/utils.h>
 #include <osmocom/sim/class_tables.h>
 
-static const uint8_t iso7816_ins_tbl[256] = {
+static const uint8_t iso7816_ins_tbl[] = {
 	[0xB0]	= 2,	/* READ BIN */
 	[0xD0]	= 3,	/* WRITE BIN */
 	[0xD6]	= 3,	/* UPDATE BIN */
@@ -178,6 +178,7 @@ static int gp_cla_ins_helper(const struct osim_cla_ins_case *cic,
 {
 	uint8_t ins = hdr[1];
 	uint8_t p1 = hdr[2];
+	uint8_t p3 = hdr[4];
 
 	switch (ins) {
 	case 0xE2:	/* STORE DATA */
@@ -187,6 +188,26 @@ static int gp_cla_ins_helper(const struct osim_cla_ins_case *cic,
 		default:
 			return 3;
 		}
+		break;
+	case 0xF2:
+		/* in their infinite wisdom, GlobalPlatform re-defined the CLA 8x / INS F2 command, so one can
+		 * take a guess if it's GlobalPlatform or ETSI.  Lucikly, the P1 coding of ETSI [so far]
+		 * states all the four upper bits must be 0, while GP always has one of those bits set */
+		if (p1 & 0xF0)
+			return 4; /* GlobalPlatform v2.2 11.4.2 */
+		else
+			return 2; /* ETSI TS 102 221 V16.2.0 11.1.2 */
+		break;
+	case 0xCA:
+	case 0xCB:
+		/* in their infinite wisdom, GlobalPlatform made GET DATA a command that can be either APDU
+		 * case 2 or case 4.  As the specify Le must be 0x00, we can conclude that P3 == 0x00 must be
+		 * Le, while P3 != 0x00 must be Lc and hence case 4 */
+		if (p3 == 0x00)
+			return 2;
+		else
+			return 4;
+		break;
 	}
 	return 0;
 }
@@ -215,14 +236,13 @@ static const uint8_t uicc_ins_tbl_80[256] = {
 static const uint8_t gp_ins_tbl_8ce[256] = {
 	[0xE4]		= 4,	/* DELETE */
 	[0xE2]		= 0x80,	/* STORE DATA */
-	[0xCA]		= 4,	/* GET DATA */
-	[0xCB]		= 4,	/* GET DATA */
-	[0xF2]		= 4,	/* GET STATUS */
+	[0xCA]		= 0x80,	/* GET DATA */
+	[0xCB]		= 0x80,	/* GET DATA */
+	[0xF2]		= 0x80,	/* GET STATUS */
 	[0xE6]		= 4,	/* INSTALL */
 	[0xE8]		= 4,	/* LOAD */
 	[0xD8]		= 4,	/* PUT KEY */
 	[0xF0]		= 3,	/* SET STATUS */
-	[0xC0]		= 2,	/* GET RESPONSE */
 };
 
 static const struct osim_cla_ins_case uicc_ins_case[] = {
@@ -245,6 +265,12 @@ static const struct osim_cla_ins_case uicc_ins_case[] = {
 		.cla_mask	= 0xF0,
 		.helper		= uicc046_cla_ins_helper,
 		.ins_tbl	= uicc_ins_tbl_046,
+	}, {
+		/* must be before uicc_ins_tbl_8ce below with same CLA+mask */
+		.cla		= 0x80,
+		.cla_mask	= 0xF0,
+		.helper		= gp_cla_ins_helper,
+		.ins_tbl	= gp_ins_tbl_8ce,
 	}, {
 		.cla		= 0x80,
 		.cla_mask	= 0xF0,
@@ -257,11 +283,6 @@ static const struct osim_cla_ins_case uicc_ins_case[] = {
 		.cla		= 0xE0,
 		.cla_mask	= 0xF0,
 		.ins_tbl	= uicc_ins_tbl_8ce,
-	}, {
-		.cla		= 0x80,
-		.cla_mask	= 0xF0,
-		.helper		= gp_cla_ins_helper,
-		.ins_tbl	= gp_ins_tbl_8ce,
 	}, {
 		.cla		= 0xC0,
 		.cla_mask	= 0xF0,
@@ -307,6 +328,12 @@ static const struct osim_cla_ins_case uicc_sim_ins_case[] = {
 		.cla_mask	= 0xF0,
 		.helper		= uicc046_cla_ins_helper,
 		.ins_tbl	= uicc_ins_tbl_046,
+	}, {
+		/* must be before uicc_ins_tbl_8ce below with same CLA+mask */
+		.cla		= 0x80,
+		.cla_mask	= 0xF0,
+		.helper		= gp_cla_ins_helper,
+		.ins_tbl	= gp_ins_tbl_8ce,
 	}, {
 		.cla		= 0x80,
 		.cla_mask	= 0xF0,
@@ -350,13 +377,6 @@ const uint8_t usim_ins_case[256] = {
 	[0x88]		= 4,	/* AUTHENTICATE */
 };
 
-/* https://learn.microsoft.com/en-us/windows-hardware/drivers/smartcard/discovery-process */
-static const uint8_t microsoft_discovery_ins_tbl[256] = {
-	[0xA4]	= 4,	/* SELECT FILE */
-	[0xCA]	= 2,	/* GET DATA */
-	[0xC0]	= 2,	/* GET RESPONSE */
-};
-
 int osim_determine_apdu_case(const struct osim_cla_ins_card_profile *prof,
 			     const uint8_t *hdr)
 {
@@ -381,16 +401,5 @@ int osim_determine_apdu_case(const struct osim_cla_ins_card_profile *prof,
 			return rc;
 		}
 	}
-	/* special handling for Microsoft who insists to use INS=0xCA in CLA=0x00 which is not
-	 * really part of GSM SIM, ETSI UICC or 3GPP USIM specifications, but only ISO7816. Rather than adding
-	 * it to each and every card profile, let's add the instructions listed at
-	 * https://learn.microsoft.com/en-us/windows-hardware/drivers/smartcard/discovery-process explicitly
-	 * here. They will only be used in case no more specific match was found in the actual profile above. */
-	if (cla == 0x00) {
-		rc = microsoft_discovery_ins_tbl[ins];
-		if (rc)
-			return rc;
-	}
-
 	return 0;
 }
